Insertion changed to match, causing misalignment (commit: 607e11d43d840ec5abbbbcf685e8b469521b91a6)

Using revision 599e8407799ec603c3d36e9275cea959ede3a0bf (1 revision before 607e...):
$ bwa aln genome.fa read.fa | bwa samse genome.fa - read.fa  > good.sam
$ cat good.sam
@SQ	SN:Genome	LN:300
Sequence	16	Genome	36	37	1I3M1D37M1D146M	*	0	0	GCAGAATTGAAGCTGCTCCTTTGAATTTGCAATTCAATGTGAAATTCACCACGGGACTTGATAAGAAGAGGATTCCAACCCCTGTCTTTAGATTTACAGTCTAATGCTTACTCAGCCATCTTACCTATGTTCATCAACCGCTGACTATTTTCAACTAACCACAAAGACATCGGCACTCTGTACCTCC	*	XT:A:U	NM:i:3	X0:i:1	X1:i:0	XM:i:4	XO:i:1	XG:i:1	MD:Z:3^A37^A146


Using revision 607e11d43d840ec5abbbbcf685e8b469521b91a6:
$ bwa aln genome.fa read.fa | bwa samse genome.fa - read.fa  > bad.sam
$ cat bad.sam
@SQ	SN:Genome	LN:300
Sequence	16	Genome	36	37	1M3M1D37M1D146M	*	0	0	GCAGAATTGAAGCTGCTCCTTTGAATTTGCAATTCAATGTGAAATTCACCACGGGACTTGATAAGAAGAGGATTCCAACCCCTGTCTTTAGATTTACAGTCTAATGCTTACTCAGCCATCTTACCTATGTTCATCAACCGCTGACTATTTTCAACTAACCACAAAGACATCGGCACTCTGTACCTCC	*	XT:A:U	NM:i:138	X0:i:1	X1:i:0	XM:i:4	XO:i:1	XG:i:1	MD:Z:0C0A0G0A0^A1T1G0A1G0C0T0G0C0T0C1T2G0A1T2G0C0A1T1C0A1T0G0T0G0A0^A2T1C0A0C1A0C0G2A0C0T1G0A0T0A1G0A1G0A0G1A0T1C1A1C3T0G0T0C0T2A0G0A0T2A0C0A0G0T0C0T0A1T0G0C0T1A0C0T0C0A0G0C1A0T0C0T1A0C1T0A0T0G0T1C0A0T0C0A1C1G0C0T0G0A0C0T0A0T3C0A1C0T0A1C1A0C0A2G0A0C0A0T0C0G1C0A0C0T0C0T0G0T0A0C1T0C1T0

