Phytophthora infestans T30-4 nuclear genome (Pi_nucl)
	http://protists.ensembl.org/Phytophthora_infestans/Info/Index
	ftp://ftp.ensemblgenomes.org/pub/protists/release-19/fasta/phytophthora_infestans/dna/Phytophthora_infestans.ASM14294v1.19.dna.toplevel.fa.gz

Phytophthora infestans Ia mitochondrial genome (Pi_mito)
	http://www.ncbi.nlm.nih.gov/nuccore/AY894835.1
